import os

import bs4
import responses
import requests
from bs4 import BeautifulSoup
from django.test import TestCase, override_settings
from wagtail.images import get_image_model
from wagtail_wordpress_import.block_builder import BlockBuilder, conf_promote_child_tags
from wagtail_wordpress_import.block_builder_defaults import (
    build_block_quote_block,
    build_form_block,
    build_heading_block,
    build_iframe_block,
    build_image_block,
    build_table_block,
    fetch_url,
    get_absolute_src,
    get_alignment_class,
    get_image_alt,
    get_image_file_name,
    image_linker,
)
from wagtail_wordpress_import.test.tests.utility_functions import (
    get_soup,
    mock_image,
    mock_pdf,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"


class TestBlockBuilderRemoveParents(TestCase):
    def setUp(self):
        raw_html_file = open(f"{FIXTURES_PATH}/raw_html.txt", "r")
        self.builder = BlockBuilder(raw_html_file, None, None)
        self.builder.promote_child_tags()
        self.output_remove_parent_tags = self.builder.soup
        self.expected_parent_name = "body"

    def test_remove_parent_tags_iframe(self):
        output = self.output_remove_parent_tags

        iframe = output.find("iframe", {"data-testing": "hasnoparent"})
        self.assertTrue(iframe.parent.name == self.expected_parent_name)

        iframe = output.find("iframe", {"data-testing": "hasparent"})
        self.assertTrue(iframe.parent.name == self.expected_parent_name)

    def test_remove_parent_tags_form(self):
        output = self.output_remove_parent_tags

        form = output.find("form", attrs={"data-testing": "hasnoparent"})
        self.assertTrue(form.parent.name == self.expected_parent_name)

        form = output.find("form", attrs={"data-testing": "hasparent"})
        self.assertTrue(form.parent.name == self.expected_parent_name)

    def test_remove_parent_tags_blockquote(self):
        output = self.output_remove_parent_tags

        blockquote = output.find("blockquote", attrs={"data-testing": "hasnoparent"})
        self.assertTrue(blockquote.parent.name == self.expected_parent_name)

        blockquote = output.find("blockquote", attrs={"data-testing": "hasparent"})
        self.assertTrue(blockquote.parent.name == self.expected_parent_name)

    def test_remove_parent_tags_wagtail_block_caption(self):
        self.builder.promote_child_tags()
        output = self.output_remove_parent_tags

        wagtail_block_captions = output.find_all("wagtail_block_caption")
        for idx, wagtail_block_caption in enumerate(wagtail_block_captions):
            with self.subTest(
                f"Checking that both wagtail_block_caption tags are top level elements fixture:{idx}"
            ):
                self.assertTrue(
                    wagtail_block_caption.parent.name == self.expected_parent_name
                )

    def test_conf_promote_child_tags(self):
        conf = conf_promote_child_tags()
        self.assertIn("iframe", conf["TAGS_TO_PROMOTE"])
        self.assertIn("form", conf["TAGS_TO_PROMOTE"])
        self.assertIn("blockquote", conf["TAGS_TO_PROMOTE"])

    def test_conf_promote_child_tags_includes_shortcodes_html_tags(self):
        conf = conf_promote_child_tags()
        self.assertIn("wagtail_block_caption", conf["TAGS_TO_PROMOTE"])


class TestBlockBuilderBlockDefaults(TestCase):
    def test_build_block_quote_block(self):
        input = """<blockquote data-testing="no-parent" cite="http://www.example.com">
        Lorem ipsum dolor sit amet.
        </blockquote>"""

        output = build_block_quote_block(get_soup(input, "html.parser"))
        self.assertEqual(output["type"], "block_quote")
        self.assertIsInstance(output["value"], dict)
        self.assertTrue(output["value"]["quote"].startswith("Lorem"))

    def test_build_form_block(self):
        input = """<form data-testing="no-parent"><button>Submit</button></form>"""

        output = build_form_block(get_soup(input, "html.parser"))
        self.assertEqual(output["type"], "raw_html")
        self.assertTrue(output["value"].startswith("<form"))

    def test_build_heading_block(self):
        input = """<h1>A heading 1</h1>"""
        soup = get_soup(input, "html.parser").find("h1")
        output = build_heading_block(soup)
        self.assertEqual(output["type"], "heading")
        self.assertEqual(output["value"]["importance"], "h1")

    def test_build_iframe_block(self):
        input = """<iframe src="https://www.youtube.com/embed/CQ7Gx8b7ac4" width="560" height="315" frameborder="0" allowfullscreen="allowfullscreen" data-testing="no-parent"></iframe>"""

        output = build_iframe_block(get_soup(input, "html.parser"))
        self.assertEqual(output["type"], "raw_html")
        self.assertTrue(output["value"].startswith("<div"))

    # work in progress
    def test_build_image_block(self):
        input = """<img src="http://www.example.com/image.jpg" />"""
        soup = get_soup(input, "html.parser")
        output = build_image_block(soup)
        self.assertEqual(output["type"], "image")
        self.assertEqual(output["value"], 1)

    def test_build_table_block(self):
        input = """<table id="tablepress-1" class="tablepress tablepress-id-1 dataTable"><caption>&#160;</caption>
            <thead>
            <tr class="row-1 odd">
            <th class="column-1 sorting_disabled" colspan="1" rowspan="1">Item</th>
            <th class="column-2 sorting_disabled" colspan="1" rowspan="1">Amount</th>
            </tr>
            </thead>
            <tfoot>
            <tr class="row-35 odd">
            <th class="column-1" colspan="1" rowspan="1">TOTAL:</th>
            <th class="column-2" colspan="1" rowspan="1">$1,127.67</th>
            </tr>
            </tfoot>
            <tbody class="row-hover">
            <tr class="row-2 even">
            <td class="column-1">Lorem 1</td>
            <td class="column-2">Lorem 1/1</td>
            </tr>
            <tr class="row-3 odd">
            <td class="column-1">Lorem 2</td>
            <td class="column-2">Lorem 2/1</td>
            </tr>
            </tbody>
            </table>"""

        output = build_table_block(get_soup(input, "html.parser"))
        self.assertEqual(output["type"], "raw_html")
        self.assertTrue(output["value"].startswith("<table"))


@override_settings(WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com")
class TestBlockBuilderBuild(TestCase):
    @responses.activate
    def setUp(self):
        responses.add(
            responses.GET,
            "https://www.example.com/images/bruno-4-runner.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        responses.add(
            responses.GET,
            "https://www.example.com/files/personal-finance-culminating-assignment.pdf",
            body=mock_pdf(),
            status=200,
            content_type="application/pdf",
        )
        raw_html_file = open(f"{FIXTURES_PATH}/raw_html.txt", "r")
        self.builder = BlockBuilder(raw_html_file, None, None)
        self.builder.promote_child_tags()
        self.blocks = self.builder.build()

    def test_rich_text_blocks_count(self):
        blocks = [
            block["type"] for block in self.blocks if block["type"] == "rich_text"
        ]
        self.assertEqual(len(blocks), 3)

    def test_blockquote_blocks_count(self):
        blocks = [
            block["type"] for block in self.blocks if block["type"] == "block_quote"
        ]
        self.assertEqual(len(blocks), 2)

    def test_raw_html_blocks_count(self):
        blocks = [block["type"] for block in self.blocks if block["type"] == "raw_html"]
        self.assertEqual(len(blocks), 5)

    def test_heading_blocks_count(self):
        blocks = [block["type"] for block in self.blocks if block["type"] == "heading"]
        self.assertEqual(len(blocks), 1)

    def test_richtext_block_6_content(self):
        """The expected content here should be lines 18 - 31 of raw_html.txt"""
        rich_text_content = BeautifulSoup(self.blocks[6]["value"], "html.parser")

        paragraph = rich_text_content.find("p")
        self.assertIsInstance(paragraph, bs4.element.Tag)

        list = rich_text_content.find("ul")
        self.assertIsInstance(list, bs4.element.Tag)

        heading_2 = rich_text_content.find("h2")
        self.assertIsInstance(heading_2, bs4.element.Tag)


@override_settings(WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com")
class TestRichTextImageLinking(TestCase):
    @responses.activate
    def test_embed_tag_generated_all_attrs(self):
        raw_html_file = """
        <p>
        <a href="https://www.example.com/images/bruno-4-runner.jpg">
        <img src="https://www.example.com/images/bruno-4-runner.jpg" alt="bruno 4 runner">
        </a>
        </p>
        """
        responses.add(
            responses.GET,
            "https://www.example.com/images/bruno-4-runner.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        output = image_linker(raw_html_file)
        image = get_image_model().objects.get(title="bruno-4-runner.jpg")
        soup = BeautifulSoup(output, "html.parser")
        embed_tag = soup.find("embed")

        with self.subTest("embed tag attr 'id' has the correct ID"):
            self.assertEqual(embed_tag.attrs["id"], str(image.id))

        with self.subTest("embed tag attr 'embedtype' has the correct type"):
            self.assertEqual(embed_tag.attrs["embedtype"], "image")

        with self.subTest("embed tag attr 'format' has correct style format"):
            self.assertEqual(embed_tag.attrs["format"], "fullwidth")

        with self.subTest("embed tag attr 'alt' has correct content"):
            self.assertEqual(embed_tag.attrs["alt"], "bruno 4 runner")

    @responses.activate
    def test_embed_tag_generated_class_align_left(self):
        raw_html_file = """
        <p>
        <a href="https://www.example.com/images/bruno-4-runner.jpg">
        <img src="https://www.example.com/images/bruno-4-runner.jpg" alt="bruno 4 runner" class="align-left">
        </a>
        </p>
        """
        responses.add(
            responses.GET,
            "https://www.example.com/images/bruno-4-runner.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        output = image_linker(raw_html_file)
        image = get_image_model().objects.get(title="bruno-4-runner.jpg")
        soup = BeautifulSoup(output, "html.parser")
        embed_tag = soup.find("embed")
        self.assertEqual(embed_tag.attrs["id"], str(image.id))
        self.assertEqual(embed_tag.attrs["embedtype"], "image")
        self.assertEqual(embed_tag.attrs["format"], "left")
        self.assertEqual(embed_tag.attrs["alt"], "bruno 4 runner")

    @responses.activate
    def test_embed_tag_generated_class_align_right(self):
        raw_html_file = """
        <p>
        <a href="https://www.example.com/images/bruno-4-runner.jpg">
        <img src="https://www.example.com/images/bruno-4-runner.jpg" alt="bruno 4 runner" class="align-right">
        </a>
        </p>
        """
        responses.add(
            responses.GET,
            "https://www.example.com/images/bruno-4-runner.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        output = image_linker(raw_html_file)
        image = get_image_model().objects.get(title="bruno-4-runner.jpg")
        soup = BeautifulSoup(output, "html.parser")
        embed_tag = soup.find("embed")
        self.assertEqual(embed_tag.attrs["id"], str(image.id))
        self.assertEqual(embed_tag.attrs["embedtype"], "image")
        self.assertEqual(embed_tag.attrs["format"], "right")
        self.assertEqual(embed_tag.attrs["alt"], "bruno 4 runner")


class TestBlockBuilderUtilityMethods(TestCase):
    def test_get_image_alt(self):
        input = get_soup(
            '<img src="fakeimage.jpg" alt="image alt" />', "html.parser"
        ).find("img")
        self.assertEqual(get_image_alt(input), "image alt")

    def test_get_image_file_name(self):
        self.assertEqual(get_image_file_name("fakeimage.jpg"), "fakeimage.jpg")
        self.assertEqual(get_image_file_name("folder/fakeimage.jpg"), "fakeimage.jpg")
        self.assertEqual(
            get_image_file_name("http://www.example.com/folder1/folder2/fakeimage.jpg"),
            "fakeimage.jpg",
        )

    def test_get_absolute_src(self):
        self.assertEqual(
            get_absolute_src("fakeimage.jpg", "http://www.example.com"),
            "http://www.example.com/fakeimage.jpg",
        )
        self.assertEqual(
            get_absolute_src("folder/fakeimage.jpg", "http://www.example.com"),
            "http://www.example.com/folder/fakeimage.jpg",
        )

    def test_get_absolute_src_without_base_url(self):
        self.assertEqual(
            get_absolute_src("folder/fakeimage.jpg"),
            "folder/fakeimage.jpg",
        )

    def test_get_abolute_src_slashes_at_start(self):
        self.assertEqual(
            get_absolute_src("//folder/fakeimage.jpg", "http://www.example.com"),
            "http://www.example.com/folder/fakeimage.jpg",
        )

    def test_get_alignment_class_align_left(self):
        soup = get_soup(
            '<img src="fakeimage.jpg" alt="image alt" class="align-left" />',
            "html.parser",
        ).find("img")
        self.assertEqual(get_alignment_class(soup), "left")

    def test_get_alignment_class_align_right(self):
        soup = get_soup(
            '<img src="fakeimage.jpg" alt="image alt" class="align-right" />',
            "html.parser",
        ).find("img")
        self.assertEqual(get_alignment_class(soup), "right")

    def test_get_alignment_class_not_present(self):
        soup = get_soup(
            '<img src="fakeimage.jpg" alt="image alt" />',
            "html.parser",
        ).find("img")
        self.assertEqual(get_alignment_class(soup), "fullwidth")


class TestBlockBuilderFetchUrlRequests(TestCase):
    def setUp(self):
        self.image_url = "http://example.com/no-image.jpg"
        self.page_url = "http://example.com/no-page.html"

    @responses.activate
    def test_fetch_url_success(self):
        responses.add(
            responses.GET,
            self.image_url,
            body=mock_image().read(),
            status=200,
            content_type="image/jpegs",
        )
        response, status, content_type = fetch_url(self.image_url)
        self.assertTrue(response.content.startswith(b"\xff\xd8"))
        self.assertTrue(status)
        self.assertTrue("image/jpeg" in content_type)

    @responses.activate
    def test_fetch_url_raises_connection_error(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.ConnectionError(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )

    @responses.activate
    def test_fetch_url_raises_http_error(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.HTTPError(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )

    @responses.activate
    def test_fetch_url_raises_request_exception(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.RequestException(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )

    @responses.activate
    def test_fetch_url_raises_read_timeout(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.ReadTimeout(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )

    @responses.activate
    def test_fetch_url_raises_timeout(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.Timeout(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )

    @responses.activate
    def test_fetch_url_raises_connect_timeout(self):
        responses.add(
            responses.GET,
            self.page_url,
            body=requests.ConnectTimeout(),
        )
        self.assertTrue(
            fetch_url(self.page_url),
            (None, True, None),
        )



import responses
from bs4 import BeautifulSoup
from django.test import TestCase
from wagtail.images import get_image_model
from wagtail_wordpress_import.prefilters.handle_shortcodes import (
    SHORTCODE_HANDLERS,
    BlockShortcodeHandler,
    CaptionHandler,
    register,
)
from wagtail_wordpress_import.test.tests.utility_functions import mock_image


class TestBlockShortcodeRegex(TestCase):
    def test_shortcode_is_found(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        handler = FooHandler()
        html = "foo[foo]baz[/foo]quux"

        match = handler._pattern.search(html)
        self.assertEqual(match.start(), 3)
        self.assertEqual(match.end(), 17)
        self.assertEqual(match.group("content"), "baz")
        self.assertEqual(match.group("attrs"), "")

    def test_multiple_shortcodes_are_found(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        handler = FooHandler()
        html = 'foo[foo]baz[/foo]quux[foo width="12"]spam[/foo]eggs'

        matches = list(handler._pattern.finditer(html))

        self.assertEqual(matches[0].start(), 3)
        self.assertEqual(matches[0].end(), 17)
        self.assertEqual(matches[0].group("content"), "baz")
        self.assertEqual(matches[0].group("attrs"), "")
        self.assertEqual(matches[1].start(), 21)
        self.assertEqual(matches[1].end(), 47)
        self.assertEqual(matches[1].group("content"), "spam")
        self.assertEqual(matches[1].group("attrs"), ' width="12"')

    def test_content_can_contain_square_brackets(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        handler = FooHandler()
        html = "embalmed ones[foo][stray dogs][/foo]"

        match = handler._pattern.search(html)

        self.assertEqual(match.start(), 13)
        self.assertEqual(match.end(), 36)
        self.assertEqual(
            match.group("content"),
            "[stray dogs]",
        )
        self.assertEqual(match.group("attrs"), "")

    def test_block_shortcode_handler_requires_the_closing_shortcode_tag(self):
        """This tests that an aside in square brackets is not accidentally matched."""

        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        handler = FooHandler()
        html = "Metasyntactic variables [foo and the like] are common placeholders"

        match = handler._pattern.search(html)
        self.assertFalse(match)


class TestShortcodeAttrubuteValidation(TestCase):
    def test_shortcode_name_not_set(self):
        class FooHandler(BlockShortcodeHandler):
            pass

        with self.assertRaises(NotImplementedError) as ctx:
            handler = FooHandler()
        self.assertEqual(
            "Create a subclass of BlockShortcodeHandler with a shortcode_name attribute",
            str(ctx.exception),
        )

    def test_shortcode_cannot_start_with_space(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = " foo"

        with self.assertRaises(ValueError) as ctx:
            handler = FooHandler()
        self.assertEqual(
            "The shortcode_name attribute must use upper or lower case letters or digits and cannot contain spaces",
            str(ctx.exception),
        )

    def test_shortcode_cannot_end_with_space(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo "

        with self.assertRaises(ValueError) as ctx:
            handler = FooHandler()
        self.assertEqual(
            "The shortcode_name attribute must use upper or lower case letters or digits and cannot contain spaces",
            str(ctx.exception),
        )

    def test_shortcode_cannot_contain_spaces(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "fo o"

        with self.assertRaises(ValueError) as ctx:
            handler = FooHandler()
        self.assertEqual(
            "The shortcode_name attribute must use upper or lower case letters or digits and cannot contain spaces",
            str(ctx.exception),
        )


class TestShortcodesSubstitution(TestCase):
    fodder = (
        "This is a block of text preceding the caption.\n\n"
        '[caption id="attachment_46162" align="aligncenter" width="600"]'
        '<img class="wp-image-46162 size-full" '
        'src="https://www.example.com/images/foo.jpg" '
        'alt="This describes the image" width="600" height="338" /> '
        "<em>[This is a caption about the image (the one above) in "
        '<a href="https//www.example.com/bar/" target="_blank" '
        'rel="noopener noreferrer">Glorious Rich Text</a>!]</em>[/caption]'
        "<strong>The following text is surrounded by strong tags.</strong>"
    )

    def test_basic(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        html = "ham[foo]eggs[/foo]spam"
        handler = FooHandler()
        html = handler.pre_filter(html)
        self.assertEqual(html, "ham<wagtail_block_foo>eggs</wagtail_block_foo>spam")

    def test_shortcode_at_the_start_of_a_string(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        html = "[foo]radiators[/foo]jam"
        handler = FooHandler()
        html = handler.pre_filter(html)
        self.assertEqual(html, "<wagtail_block_foo>radiators</wagtail_block_foo>jam")

    def test_beautifulsoup_can_parse(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        html = "ham[foo]eggs[/foo]spam"
        handler = FooHandler()
        html = handler.pre_filter(html)
        soup = BeautifulSoup(html, "html.parser")
        tags = soup.find_all(True)
        self.assertEqual(len(tags), 1)
        self.assertEqual(str(tags[0]), "<wagtail_block_foo>eggs</wagtail_block_foo>")

    def test_beautifulsoup_can_parse_attrs(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        html = 'ham[foo quantity=3 state="over easy" garnish="more spam"]eggs[/foo]spam'
        handler = FooHandler()
        html = handler.pre_filter(html)
        soup = BeautifulSoup(html, "html.parser")
        tags = soup.find_all(True)
        self.assertEqual(len(tags), 1)
        self.assertEqual(tags[0]["quantity"], "3")
        self.assertEqual(tags[0]["state"], "over easy")
        self.assertEqual(tags[0]["garnish"], "more spam")

    def test_caption(self):
        class CaptionHandler(BlockShortcodeHandler):
            shortcode_name = "caption"
            custom_html_tag_prefix = "wagtail_block_"

        html = 'Some pretext[caption width="100"]The content of the tag[/caption]'
        handler = CaptionHandler()
        html = handler.pre_filter(html)
        self.assertEqual(
            html,
            'Some pretext<wagtail_block_caption width="100">The content of the tag</wagtail_block_caption>',
        )

    def test_unclosed_shortcode_rejected_by_block_shortcode_handler(self):
        """This tests that an aside in square brackets is not accidentally matched."""

        class CaptionHandler(BlockShortcodeHandler):
            shortcode_name = "caption"
            custom_html_tag_prefix = "wagtail_block_"

        original = "Some pretext [caption this however you want] "
        handler = CaptionHandler()
        html = handler.pre_filter(original)
        self.assertEqual(html, original)

    def test_known_content(self):
        class CaptionHandler(BlockShortcodeHandler):
            shortcode_name = "caption"
            custom_html_tag_prefix = "wagtail_block_"

        html = (
            "This is a block of text preceding the caption.\n\n"
            '[caption id="attachment_46162" align="aligncenter" width="600"]'
            '<img class="wp-image-46162 size-full" '
            'src="https://www.example.com/images/foo.jpg" '
            'alt="This describes the image" width="600" height="338" /> '
            "<em>[This is a caption about the image (the one above) in "
            '<a href="https//www.example.com/bar/" target="_blank" '
            'rel="noopener noreferrer">Glorious Rich Text</a>!]</em>'
            "[/caption]"
            "<strong>The following text is surrounded by strong tags.</strong>"
        )
        handler = CaptionHandler()
        html = handler.pre_filter(html)
        self.assertEqual(
            html,
            "This is a block of text preceding the caption.\n\n"
            '<wagtail_block_caption id="attachment_46162" align="aligncenter" width="600">'
            '<img class="wp-image-46162 size-full" '
            'src="https://www.example.com/images/foo.jpg" '
            'alt="This describes the image" width="600" height="338" /> '
            "<em>[This is a caption about the image (the one above) in "
            '<a href="https//www.example.com/bar/" target="_blank" '
            'rel="noopener noreferrer">Glorious Rich Text</a>!]</em>'
            "</wagtail_block_caption>"
            "<strong>The following text is surrounded by strong tags.</strong>",
        )


class TestAbsentShortcodeHandlers(TestCase):
    def test_included_shortcodes(self):
        # prime the SHORTCODE_HANDLERS
        # note this class has not been registered
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        caption_handler_instance = SHORTCODE_HANDLERS[0]()

        self.assertIsInstance(caption_handler_instance, CaptionHandler)
        self.assertNotIsInstance(caption_handler_instance, FooHandler)
        self.assertEqual(len(SHORTCODE_HANDLERS), 1)


class TestIncludedShortcodeHandlers(TestCase):
    def test_included_shortcodes(self):
        # prime the SHORTCODE_HANDLERS
        # note this class has been registered
        @register()
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            custom_html_tag_prefix = "wagtail_block_"

        caption_handler_instance = SHORTCODE_HANDLERS[0]()
        foo_handler_instance = SHORTCODE_HANDLERS[1]()

        self.assertIsInstance(caption_handler_instance, CaptionHandler)
        self.assertIsInstance(foo_handler_instance, FooHandler)
        self.assertEqual(len(SHORTCODE_HANDLERS), 2)


class TestProvidedShortcodeIsTopLevel(TestCase):
    """
    Block Shortcode Handler has a is_top_level_html_tag property that is True by default.
    It can be set to False to indicate that the shortcode is not a top level HTML tag.
    """

    def test_is_top_level_html_tag_defaults_to_true(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"

        foo = FooHandler()
        self.assertTrue(foo.is_top_level_html_tag)

    def test_is_top_level_html_tag_can_be_overridden(self):
        class FooHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
            is_top_level_html_tag = False

        foo = FooHandler()
        self.assertFalse(foo.is_top_level_html_tag)


class TestShortcodeHandlerStreamfieldBlockCreation(TestCase):
    @responses.activate
    def test_construct_block_method_output(self):
        responses.add(
            responses.GET,
            "https://www.example.com/images/j-money-family-portrait.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        handler = CaptionHandler()
        wagtail_custom_html = BeautifulSoup(
            '<wagtail_block_caption id="46162" align="aligncenter" width="600">'
            '<img class="size-full" '
            'src="https://www.example.com/images/j-money-family-portrait.jpg" '
            'alt="This describes the image" width="600" height="338" /> '
            "<em>[This is a caption about the image (the one above) in "
            '<a href="https//www.example.com/bar/" target="_blank" '
            'rel="noopener noreferrer">Glorious Rich Text</a>!]</em>'
            "</wagtail_block_caption>",
            "html.parser",
        )
        json = handler.construct_block(
            wagtail_custom_html.find("wagtail_block_caption")
        )
        image = get_image_model().objects.get(title="j-money-family-portrait.jpg")

        self.assertIsInstance(json, dict)
        self.assertEqual(json["type"], "image")
        self.assertEqual(json["value"]["image"], image.id)
        self.assertEqual(
            json["value"]["caption"],
            "[This is a caption about the image (the one above) in Glorious Rich Text!]",
        )
        self.assertEqual(json["value"]["alignment"], "center")
        self.assertEqual(json["value"]["link"], "https//www.example.com/bar/")


class TestCaptionHandler(TestCase):
    @responses.activate
    def test_absence_of_image(self):
        handler = CaptionHandler()
        wagtail_custom_html = """
        <wagtail_block_caption id="attachment_46162" align="aligncenter" width="600">
            <a href="http://www.example.com/">
                
            </a>This is a caption about the image
        </wagtail_block_caption>"""

        output = handler.construct_block(
            BeautifulSoup(wagtail_custom_html, "html.parser").find(
                f"wagtail_block_{handler.shortcode_name}"
            )
        )
        # The image is not present in the html, the output should be a raw_html block
        self.assertEqual(output["type"], "raw_html")
        self.assertTrue("No image found in caption" in output["value"])

    @responses.activate
    def test_construct_block_method_output(self):
        handler = CaptionHandler()
        responses.add(
            responses.GET,
            "https://www.example.com/images/foo.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        wagtail_custom_html = """
        <wagtail_block_caption id="attachment_46162" align="aligncenter" width="600">
            <a href="http://www.example.com/">
                <img
                    class="wp-image-46162 size-full" 
                    src="https://www.example.com/images/foo.jpg" 
                    alt="This describes the image" 
                    width="600" 
                    height="338" />
            </a>This is a caption about the image
        </wagtail_block_caption>"""
        output = handler.construct_block(
            BeautifulSoup(wagtail_custom_html, "html.parser").find(
                f"wagtail_block_{handler.shortcode_name}"
            )
        )
        image = get_image_model().objects.get(title="foo.jpg")
        self.assertEqual(output["type"], "image")
        self.assertEqual(output["value"]["image"], image.id)
        self.assertEqual(
            output["value"]["caption"], "This is a caption about the image"
        )
        self.assertEqual(output["value"]["alignment"], "center")
        self.assertEqual(output["value"]["link"], "http://www.example.com/")

    @responses.activate
    def test_absence_of_alignment(self):
        handler = CaptionHandler()
        responses.add(
            responses.GET,
            "https://www.example.com/images/foo.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        wagtail_custom_html = """
        <wagtail_block_caption id="attachment_46162" width="600">
            <a href="http://www.example.com/">
                <img
                    class="wp-image-46162 size-full"
                    src="https://www.example.com/images/foo.jpg"
                    alt="This describes the image"
                    width="600"
                    height="338" />
            </a>This is a caption about the image
        </wagtail_block_caption>"""

        output = handler.construct_block(
            BeautifulSoup(wagtail_custom_html, "html.parser").find(
                f"wagtail_block_{handler.shortcode_name}"
            )
        )
        image = get_image_model().objects.get(title="foo.jpg")
        self.assertEqual(output["type"], "image")
        self.assertEqual(output["value"]["image"], image.id)
        self.assertEqual(
            output["value"]["caption"], "This is a caption about the image"
        )
        self.assertEqual(output["value"]["alignment"], "left")
        self.assertEqual(output["value"]["link"], "http://www.example.com/")

    @responses.activate
    def test_absence_of_an_anchor(self):
        handler = CaptionHandler()
        responses.add(
            responses.GET,
            "https://www.example.com/images/foo.jpg",
            body=mock_image().read(),
            status=200,
            content_type="image/jpeg",
        )
        wagtail_custom_html = """
        <wagtail_block_caption id="attachment_46162" align="aligncenter" width="600">
                <img
                    class="wp-image-46162 size-full" 
                    src="https://www.example.com/images/foo.jpg" 
                    alt="This describes the image" 
                    width="600" 
                    height="338" />
            This is a caption about the image
        </wagtail_block_caption>"""
        output = handler.construct_block(
            BeautifulSoup(wagtail_custom_html, "html.parser").find(
                f"wagtail_block_{handler.shortcode_name}"
            )
        )
        image = get_image_model().objects.get(title="foo.jpg")
        self.assertEqual(output["type"], "image")
        self.assertEqual(output["value"]["image"], image.id)
        self.assertEqual(
            output["value"]["caption"], "This is a caption about the image"
        )
        self.assertEqual(output["value"]["alignment"], "center")
        self.assertEqual(output["value"]["link"], "")



#!/usr/bin/env python

from os import path

from setuptools import find_packages, setup

from wagtail_wordpress_import import __version__


this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "docs/long_description.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="wagtail-wordpress-import",
    version=__version__,
    description="Import XML data into wagtail to create pages and content",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Nick Moreton",
    author_email="nickmoreton@me.com",
    url="https://github.com/torchbox/wagtail-wordpress-import",
    packages=find_packages(),
    include_package_data=True,
    license="BSD",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Framework :: Django",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 2",
    ],
    install_requires=[
        "Django>=3.1,<3.3",
        "Wagtail>=2.14,<2.16",
        "lxml>=4.7,<4.8",
        "bleach>=4.1,<4.2",
        "prettytable>=2.2,<2.3",
        "shortcodes>=5.1,<6.0",
    ],
    extras_require={
        "testing": [
            "dj-database-url==0.5.0",
            "freezegun==0.3.15",
            "responses>=0.16.0,<0.17.0",
        ],
    },
    zip_safe=False,
)



import re

from bs4 import BeautifulSoup
from django.utils.module_loading import import_string
from wagtail_wordpress_import.prefilters.transform_styles_defaults import (
    HTML_TAGS,
    conf_transform_html_tags_enabled,
    conf_transform_html_tags_mapping,
    transform_float_left,
    transform_float_right,
    transform_style_bold,
    transform_style_bold_italic,
    transform_style_center,
    transform_style_italic,
    transform_style_left,
    transform_style_right,
)

CONF_STYLES_MAPPING = [
    (
        re.compile(r"font-style:italic;font-weight:bold;", re.IGNORECASE),
        transform_style_bold_italic,
    ),
    (re.compile(r"font-weight:bold;", re.IGNORECASE), transform_style_bold),
    (re.compile(r"font-style:italic;", re.IGNORECASE), transform_style_italic),
    (
        re.compile(
            r"text-align:center;",
            re.IGNORECASE,
        ),
        transform_style_center,
    ),
    (re.compile(r"text-align:left;", re.IGNORECASE), transform_style_left),
    (re.compile(r"text-align:right;", re.IGNORECASE), transform_style_right),
    (re.compile(r"float:left;", re.IGNORECASE), transform_float_left),
    (re.compile(r"float:right;", re.IGNORECASE), transform_float_right),
]


def normalize_style_attrs(soup):
    """
    Normalize the style attrs on tags so we have a predictable format for parsing.

    e.g. font-style: italic becomes font-style:italic;
    e.g. FONT-WEIGHT:400; becomes font-weight:400;
    e.g. font-weight: bold;font-style:italic; becomes font-style:italic;font-weight:bold;
    The style rules are all lowercased, with `;` appended and have no spaces.
    Spaces are removed between multiple style rules.
    Style rules are sorted alphabetically

    The styles attrs won't exist in the final page content they are removed later on
    in filter_bleach_clean() method, they are kept around for any other processing needs.
    """

    elements = soup.findAll(recursive=True)

    for el in elements:
        if el.attrs and el.attrs.get("style"):
            styles_list = [
                style.strip().lower().replace(" ", "") + ";"
                for style in el.attrs["style"].split(";")
                if style != ""
            ]
            el.attrs["style"] = "".join(sorted(styles_list))

    return soup


def filter_transform_inline_styles(html, options=None):
    """
    Use the default or provided CONFIG to loop through each filter
    and apply the transform_* method

    params
        html: raw html input, the input needs to be valid html
        options: allows a developer to override the default config
        and pass in HTML_TAGS and TRANSFORM_STYLES_MAPPING
    """

    soup = normalize_style_attrs(BeautifulSoup(html, "html.parser"))

    CONF_HTML_TAGS = HTML_TAGS
    if options and options.get("HTML_TAGS"):
        html_tags = import_string(options["HTML_TAGS"])
        if callable(html_tags):
            CONF_HTML_TAGS = html_tags()
        else:
            CONF_HTML_TAGS = html_tags

    styles_mapping = CONF_STYLES_MAPPING

    if options and "TRANSFORM_STYLES_MAPPING" in options:
        styles_mapping = options["TRANSFORM_STYLES_MAPPING"]
        for filter in styles_mapping:
            filter_transform_styles(
                soup.findAll(style=filter[0]),
                soup,
                import_string(filter[1]),
                CONF_HTML_TAGS,
            )
    else:
        for filter in styles_mapping:
            filter_transform_styles(
                soup.findAll(style=filter[0]), soup, filter[1], CONF_HTML_TAGS
            )

    if conf_transform_html_tags_enabled():
        for filter in conf_transform_html_tags_mapping():
            tags = soup.findAll(filter[0])

            for tag in tags:
                filter[1](soup, tag)

    return str(soup)


def filter_transform_styles(tags, soup, filter_method, conf_html_tags):
    for tag in tags:
        if tag.name not in conf_html_tags:
            print("item.name = tag not found in HTML_TAGS")

        filter_method(soup, tag)



from django.apps import AppConfig


class WagtailWordpressImportTestAppConfig(AppConfig):
    label = "wagtail_wordpress_import_test"
    name = "wagtail_wordpress_import.test"
    verbose_name = "Wagtail wordpress import tests"



import json
import os
import re
import unittest
from collections import Counter
from datetime import datetime
from unittest import mock
from xml.dom import pulldom

from django.test import TestCase, override_settings
from wagtail.core.models import Page

from example.models import Category
from wagtail_wordpress_import.functions import node_to_dict
from wagtail_wordpress_import.importers.wordpress import (
    DEFAULT_PREFILTERS,
    WordpressImporter,
    WordpressItem,
)
from wagtail_wordpress_import.logger import Logger

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"
LOG_DIR = "fakedir"
IMPORTER_RUN_PARAMS_TEST = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["post", "page"],
    "page_statuses": ["publish", "draft"],
}


class WordpressItemTests(TestCase):
    def setUp(self):
        self.logger = Logger("fakedir")
        body_html = """<p>Dummmy text</p><p>Dummmy text</p><p>Dummmy text</p>"""
        self.good_node = {
            "title": "Page Title",
            "wp:post_name": "page-title",
            "wp:post_date_gmt": "2017-03-12 17:53:57",
            "wp:post_modified_gmt": "2018-12-04 11:49:24",
            "content:encoded": body_html,
            "wp:post_id": "1000",
            "wp:post_type": "post",
            "link": "http://www.example.com",
        }
        self.bad_node = {
            "title": "Page Title",
            "wp:post_name": "",
            "wp:post_date_gmt": "0000-00-00 00:00:00",
            "wp:post_modified_gmt": "0000-00-00 00:00:00",
            "content:encoded": body_html,
            "wp:post_id": "1000",
            "wp:post_type": "post",
            "link": "",
        }
        self.no_title = {
            "wp:post_name": "no-title",
            "wp:post_date_gmt": "2017-03-12 17:53:57",
            "wp:post_modified_gmt": "2018-12-04 11:49:24",
            "content:encoded": body_html,
            "wp:post_id": "1000",
            "wp:post_type": "post",
            "link": "http://www.example.com",
        }

    def test_all_fields_with_good_data(self):
        wordpress_item = WordpressItem(self.good_node, self.logger)
        title = wordpress_item.cleaned_data["title"]
        slug = wordpress_item.cleaned_data["slug"]
        first_published_at = wordpress_item.cleaned_data["first_published_at"]
        last_published_at = wordpress_item.cleaned_data["last_published_at"]
        latest_revision_created_at = wordpress_item.cleaned_data[
            "latest_revision_created_at"
        ]
        body = wordpress_item.cleaned_data["body"]
        wp_post_id = wordpress_item.cleaned_data["wp_post_id"]
        wp_post_type = wordpress_item.cleaned_data["wp_post_type"]
        wp_link = wordpress_item.cleaned_data["wp_link"]
        wp_raw_content = wordpress_item.debug_content["filter_linebreaks_wp"]
        wp_processed_content = wordpress_item.debug_content[
            "filter_transform_inline_styles"
        ]
        wp_block_json = wordpress_item.debug_content["block_json"]

        self.assertEqual(title, "Page Title")
        self.assertEqual(slug, "page-title")
        self.assertIsInstance(first_published_at, datetime)
        self.assertIsInstance(last_published_at, datetime)
        self.assertIsInstance(latest_revision_created_at, datetime)
        self.assertIsInstance(json.dumps(body), str)
        self.assertEqual(wp_post_id, 1000)
        self.assertEqual(wp_post_type, "post")
        self.assertEqual(wp_link, "http://www.example.com")
        self.assertIsInstance(wp_raw_content, str)
        self.assertIsInstance(wp_processed_content, str)
        self.assertIsInstance(wp_block_json, list)
        self.assertTrue(
            len(wp_block_json), 1
        )  # we are only parsing consecutive paragraphs so the will only be one block (rich_text)

    def test_cleaned_fields(self):
        wordpress_item = WordpressItem(self.bad_node, self.logger)
        slug = wordpress_item.cleaned_data["slug"]
        first_published_at = wordpress_item.cleaned_data["first_published_at"]
        last_published_at = wordpress_item.cleaned_data["last_published_at"]
        latest_revision_created_at = wordpress_item.cleaned_data[
            "latest_revision_created_at"
        ]
        wp_link = wordpress_item.cleaned_data["wp_link"]
        self.assertEqual(slug, "page-title")
        self.assertIsInstance(first_published_at, datetime)
        self.assertIsInstance(last_published_at, datetime)
        self.assertIsInstance(latest_revision_created_at, datetime)
        self.assertEqual(wp_link, "")

    def test_no_title_faked(self):
        wordpress_item = WordpressItem(self.no_title, self.logger)
        title = wordpress_item.cleaned_data["title"]

        self.assertEqual(
            self.no_title["wp:post_id"], "1000"
        )  # just a fixture regression test
        self.assertEqual(title, f"no-title-available-1000")


@override_settings(
    WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://localhost:8000",
    WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_ENABLED=True,
    WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_MODEL="example.models.Category",
)  # testing requires a live domain for requests to use, this is something I need to change before package release
# mocking of somesort, using localhost:8000 for now
class WordpressItemImportTests(TestCase):
    from example.models import Category

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST["parent_id"],
            page_types=IMPORTER_RUN_PARAMS_TEST["page_types"],
            page_statuses=IMPORTER_RUN_PARAMS_TEST["page_statuses"],
        )

        self.parent_page = Page.objects.get(id=IMPORTER_RUN_PARAMS_TEST["parent_id"])
        self.imported_pages = self.parent_page.get_children().all()

    def test_category_snippets_are_saved(self):
        snippets = Category.objects.all()
        self.assertEqual(len(snippets), 4)

    def test_page_one_has_categories(self):
        page_one = self.imported_pages.get(title="Item one title")
        categories = page_one.specific.categories.all()
        self.assertEqual(2, categories.count())
        self.assertEqual(categories[0].name, "Blogging")
        self.assertEqual(categories[1].name, "Life")

    def test_page_two_has_categories(self):
        page_two = self.imported_pages.get(title="Item two title")
        categories = page_two.specific.categories.all()
        self.assertEqual(3, categories.count())
        self.assertEqual(categories[0].name, "Blogging")
        self.assertEqual(categories[1].name, "Cars")
        self.assertEqual(categories[2].name, "Computing")

    def test_short_category_is_not_imported(self):
        page_one = self.imported_pages.get(title="Item one title")
        categories = [category.name for category in page_one.specific.categories.all()]
        self.assertNotIn("A", categories)

    def test_categories_have_no_duplicate_entries(self):
        categories = [category.name for category in Category.objects.all()]
        duplicates = [
            k for k, v in Counter(categories).items() if v > 1
        ]  # duplicates will be empty if no duplicate category names exist
        self.assertEqual(len(duplicates), 0)


@override_settings(
    WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://localhost:8000",
    WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_ENABLED=True,
    WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_MODEL="example.models.Category",
)  # testing requires a live domain for requests to use, this is something I need to change before package release
# mocking of somesort, using localhost:8000 for now
class WordpressItemImportTestsNoCategories(TestCase):
    from example.models import Category

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST["parent_id"],
            page_types=["hasnocategories"],
            page_statuses=["hasnocategories"],
        )

        self.parent_page = Page.objects.get(id=IMPORTER_RUN_PARAMS_TEST["parent_id"])
        self.imported_pages = self.parent_page.get_children().all()

    def test_page_has_no_categories(self):
        page = self.imported_pages.first()
        categories = page.specific.categories.all()
        self.assertEqual(0, categories.count())

    def test_categories_count_is_zero(self):
        count = Category.objects.count()
        self.assertEqual(count, 0)


IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1 = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["post"],
    "page_statuses": ["publish"],
}


@override_settings(
    WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_ENABLED=True,
)
class WordpressImporterTestsYoastMetaDescriptions(TestCase):
    """
    This tests when a wp:postmeta for none single or multiple keys in the XML file.
    If the meta key for yoast is not present the <description></description> content is returned.
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.logger = Logger("fakedir")
        xml_file = open(f"{FIXTURES_PATH}/post_meta.xml", "rb")
        xml_doc = pulldom.parse(xml_file)
        self.items_dict = []
        for event, node in xml_doc:
            if event == pulldom.START_ELEMENT and node.tagName == "item":
                xml_doc.expandNode(node)
                self.items_dict.append(node_to_dict(node))

    def test_items_dict_0(self):
        # self.items_dict[0] = the single item wp:post_meta without yoast
        wordpress_item = WordpressItem(self.items_dict[0], self.logger)
        self.assertEqual(
            wordpress_item.get_yoast_description_value(),
            "This page has a default description",
        )

    def test_items_dict_1(self):
        # self.items_dict[1] = the multiple item wp:post_meta
        wordpress_item = WordpressItem(self.items_dict[1], self.logger)
        self.assertEqual(
            wordpress_item.get_yoast_description_value(),
            "This page has a default description",
        )

    def test_items_dict_2(self):
        # self.items_dict[2] = the single item wp:post_meta with yoast
        wordpress_item = WordpressItem(self.items_dict[2], self.logger)
        self.assertEqual(
            wordpress_item.get_yoast_description_value(),
            "This is a yoast metadesc!",
        )

    def test_items_dict_3(self):
        # self.items_dict[3] = the multiple item wp:post_meta with yoast
        wordpress_item = WordpressItem(self.items_dict[3], self.logger)
        self.assertEqual(
            wordpress_item.get_yoast_description_value(),
            "This is a yoast metadesc!",
        )

    def test_items_dict_4(self):
        # self.items_dict[3] = the multiple item wp:post_meta with yoast
        wordpress_item = WordpressItem(self.items_dict[4], self.logger)
        self.assertEqual(
            wordpress_item.get_yoast_description_value(),
            "This page has a default description",
        )


class WordpressImporterTestsCleanWpPostMeta(TestCase):
    """
    This tests the wp_post_meta field contents after cleaning in
    WordpressItem().clean_wp_post_meta()
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.logger = Logger("fakedir")
        xml_file = open(f"{FIXTURES_PATH}/post_meta.xml", "rb")
        xml_doc = pulldom.parse(xml_file)
        self.items_dict = []
        for event, node in xml_doc:
            if event == pulldom.START_ELEMENT and node.tagName == "item":
                xml_doc.expandNode(node)
                self.items_dict.append(node_to_dict(node))

    def test_items_dict_0(self):
        # self.items_dict[0] = the single item wp:post_meta without yoast
        wordpress_item = WordpressItem(self.items_dict[0], self.logger)
        thumbnail_id = wordpress_item.clean_wp_post_meta()["thumbnail_id"]
        self.assertEqual(thumbnail_id, 43124)

    def test_items_dict_1(self):
        # self.items_dict[1] = the multiple item wp:post_meta
        wordpress_item = WordpressItem(self.items_dict[1], self.logger)
        post_meta = wordpress_item.clean_wp_post_meta()
        self.assertEqual(post_meta["facebook_shares"], 100)
        self.assertEqual(post_meta["pinterest_shares"], 200)
        self.assertEqual(post_meta["twitter_shares"], 300)

    def test_items_dict_2(self):
        # self.items_dict[2] = the single item wp:post_meta with yoast
        wordpress_item = WordpressItem(self.items_dict[2], self.logger)
        post_meta = wordpress_item.clean_wp_post_meta()
        self.assertEqual(post_meta["yoast_wpseo_metadesc"], "This is a yoast metadesc!")

    def test_items_dict_3(self):
        # self.items_dict[3] = the multiple item wp:post_meta with yoast
        wordpress_item = WordpressItem(self.items_dict[3], self.logger)
        post_meta = wordpress_item.clean_wp_post_meta()
        self.assertEqual(post_meta["facebook_shares"], 10)
        self.assertEqual(post_meta["pinterest_shares"], 20)
        self.assertEqual(post_meta["twitter_shares"], 30)
        self.assertEqual(post_meta["yoast_wpseo_metadesc"], "This is a yoast metadesc!")

    def test_items_dict_4(self):
        # self.items_dict[4] = has no wp:post_meta items
        wordpress_item = WordpressItem(self.items_dict[4], self.logger)
        with self.assertRaises(KeyError):
            wordpress_item.clean_wp_post_meta()["wp:postmeta"]

    def test_items_dict_1_excluded_keys(self):
        wordpress_item = WordpressItem(self.items_dict[1], self.logger)
        cleaned_postmeta = wordpress_item.clean_wp_post_meta()
        with self.assertRaises(KeyError):
            cleaned_postmeta["wp:postmeta"]
        with self.assertRaises(KeyError):
            cleaned_postmeta["wp_post_meta"]
        with self.assertRaises(KeyError):
            cleaned_postmeta["content:encoded"]
        with self.assertRaises(KeyError):
            cleaned_postmeta["dc:creator"]
        with self.assertRaises(KeyError):
            cleaned_postmeta["wp:post_id"]

    def test_items_dict_1_included_keys(self):
        wordpress_item = WordpressItem(self.items_dict[1], self.logger)
        cleaned_postmeta = wordpress_item.clean_wp_post_meta()
        self.assertTrue("title" in cleaned_postmeta)
        self.assertTrue("dc_creator" in cleaned_postmeta)
        self.assertTrue("guid" in cleaned_postmeta)
        self.assertTrue("description" in cleaned_postmeta)
        self.assertTrue("wp_post_id" in cleaned_postmeta)
        self.assertTrue("wp_post_date" in cleaned_postmeta)
        self.assertTrue("category" in cleaned_postmeta)
        self.assertTrue("facebook_shares" in cleaned_postmeta)
        self.assertTrue("pinterest_shares" in cleaned_postmeta)
        self.assertTrue("twitter_shares" in cleaned_postmeta)


class TestWordpressItemPrefilterConfig(TestCase):
    def test_prefilter_content_default(self):
        # The expected output should be transformed after passing through the
        # the default prefilters
        node = {"content:encoded": "foo bar baz"}
        wordpress_item = WordpressItem(node, "")
        output = wordpress_item.prefilter_content(wordpress_item.raw_body)
        self.assertEqual(output, "<p>foo bar baz</p>\n")


class TestWordpressPrefilterDefaults(TestCase):
    def test_default_prefilters(self):
        self.assertIsInstance(DEFAULT_PREFILTERS, list)
        self.assertTrue(len(DEFAULT_PREFILTERS), 4)
        self.assertEqual(
            DEFAULT_PREFILTERS[0]["FUNCTION"],
            "wagtail_wordpress_import.prefilters.linebreaks_wp",
        )
        self.assertEqual(
            DEFAULT_PREFILTERS[1]["FUNCTION"],
            "wagtail_wordpress_import.prefilters.transform_shortcodes",
        )
        self.assertEqual(
            DEFAULT_PREFILTERS[2]["FUNCTION"],
            "wagtail_wordpress_import.prefilters.transform_inline_styles",
        )
        self.assertEqual(
            DEFAULT_PREFILTERS[3]["FUNCTION"],
            "wagtail_wordpress_import.prefilters.bleach_clean",
        )


def foo_filter(content, options):
    return content, options


def transform_foo(soup, tag):
    new_tag = soup.new_tag("foo")
    new_tag.string = tag.string
    tag.replace_with(new_tag)


class TestWordpressItemPrefilterOverride(TestCase):
    """Test developers' ability to edit settings.WAGTAIL_WORDPRESS_IMPORT_PREFILTERS"""

    @override_settings(WAGTAIL_WORDPRESS_IMPORT_PREFILTERS=[])
    def test_prefilter_content_no_filters(self):
        """Remove all pre-filters

        The expected output is the same as the input because there are no prefilters to
        apply to the content
        """
        node = {"content:encoded": "foo bar baz"}
        wordpress_item = WordpressItem(node, "")
        output = wordpress_item.prefilter_content(wordpress_item.raw_body)
        self.assertEqual(output, "foo bar baz")

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_PREFILTERS=[
            {
                "FUNCTION": "wagtail_wordpress_import.test.tests.test_wordpress_item.foo_filter"
            }
        ]
    )
    def test_custom_provided_prefilter(self):
        """Provide a custom pre-filter

        The expected output is the same as the input because the applied filters do
        nothing and return the same value.
        """
        node = {"content:encoded": "foo bar baz"}
        wordpress_item = WordpressItem(node, "")
        output = wordpress_item.prefilter_content(wordpress_item.raw_body)
        self.assertEqual(output[0], "foo bar baz")
        self.assertEqual(output[1], None)

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_PREFILTERS=[
            {
                "FUNCTION": "wagtail_wordpress_import.test.tests.test_wordpress_item.foo_filter",
                "OPTIONS": {"foo": "bar"},
            }
        ]
    )
    def test_custom_provided_prefilter_with_options(self):
        """Provide a custom pre-filter with options

        The expected output is the same as the input because the applied filters do
        nothing and return the same value.
        """
        node = {"content:encoded": "foo bar baz"}
        wordpress_item = WordpressItem(node, "")
        output = wordpress_item.prefilter_content(wordpress_item.raw_body)
        self.assertEqual(output[0], "foo bar baz")
        self.assertEqual(output[1], {"foo": "bar"})

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_PREFILTERS=[
            {
                "FUNCTION": "wagtail_wordpress_import.prefilters.transform_inline_styles",
                "OPTIONS": {
                    "TRANSFORM_STYLES_MAPPING": [
                        (
                            re.compile(r"font-weight:bold", re.IGNORECASE),
                            "wagtail_wordpress_import.test.tests.test_wordpress_item.transform_foo",
                        )
                    ],
                },
            },
        ]
    )
    def test_transform_styles_filter_add_options(self):
        """Test that a developer can pass custom OPTIONS to transform_inline_styles.

        Here WAGTAIL_WORDPRESS_IMPORT_PREFILTERS contains only config for
        transform_inline_styles, so that other prefilters are not run, and it's easier
        to test the output.
        """
        node = {"content:encoded": '<p style="font-weight: bold">foo bar baz</p>'}
        wordpress_item = WordpressItem(node, "")
        output = wordpress_item.prefilter_content(wordpress_item.raw_body)
        self.assertEqual(output.strip(), "<foo>foo bar baz</foo>")



from wagtail.core import blocks
from wagtail.images.blocks import ImageChooserBlock


class HeadingBlock(blocks.StructBlock):
    text = blocks.CharBlock(classname="title")
    importance = blocks.ChoiceBlock(
        choices=(
            ("h1", "H1"),
            ("h2", "H2"),
            ("h3", "H3"),
            ("h4", "H4"),
            ("h5", "H5"),
            ("h6", "H6"),
        ),
        default="h1",
    )

    class Meta:
        icon = "title"
        template = "wagtail_wordpress_import/heading_block.html"


class ImageBlock(blocks.StructBlock):
    image = ImageChooserBlock()
    caption = blocks.CharBlock(required=False)
    link = blocks.URLBlock(required=False)
    alignment = blocks.ChoiceBlock(
        choices=(
            ("left", "Left"),
            ("right", "Right"),
            ("center", "Center"),
        ),
        default="left",
    )

    class Meta:
        icon = "image"
        template = "wagtail_wordpress_import/image_block.html"


class QuoteBlock(blocks.StructBlock):
    quote = blocks.CharBlock(form_classname="title")
    attribution = blocks.CharBlock(required=False)

    class Meta:
        icon = "openquote"
        template = "wagtail_wordpress_import/quote_block.html"


class WPImportStreamBlocks(blocks.StreamBlock):
    rich_text = blocks.RichTextBlock()
    heading = HeadingBlock()
    image = ImageBlock()
    block_quote = QuoteBlock()
    raw_html = blocks.RawHTMLBlock()



import os

from django.core.management import CommandError, call_command
from django.test import TestCase, override_settings
from wagtail.core.models import Page
from wagtail_wordpress_import.management.commands.reduce_xml import Command as ReduceCmd, generate_stats_file
from wagtail_wordpress_import.xml_boilerplate import (
    build_xml_stream,
    generate_temporary_file,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"


class TestImportXmlCommandNoConfig(TestCase):
    def test_without_arguments(self):
        with self.assertRaises(CommandError) as ctx:
            call_command("import_xml")
        self.assertEqual(
            str(ctx.exception),
            "Error: the following arguments are required: xml_file, parent_id",
        )

    def test_with_only_xml_file_argument(self):
        with self.assertRaises(CommandError) as ctx:
            call_command("import_xml", "test.xml")
        self.assertEqual(
            str(ctx.exception),
            "Error: the following arguments are required: parent_id",
        )

    def test_with_required_arguments(self):
        with self.assertRaises(SystemExit):
            call_command("import_xml", "test.xml", "2")


@override_settings(
    WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com",
)
class TestImportXmlCommandWithConfig(TestCase):
    def test_xml_file_error(self):
        # The XML file does not exist
        with self.assertRaises(SystemExit):
            call_command("import_xml", "test.xml", "2")

    def test_run_with_xml_and_parent_id(self):
        built_file = generate_temporary_file(
            build_xml_stream(xml_tags_fragment="").read()
        )

        with self.assertRaises(FileNotFoundError):
            # In testing there is no log folder
            # the command should raise this as an error
            call_command(
                "import_xml", built_file, "2", "-a", "example", "-m", "TestPage"
            )


@override_settings(
    WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com",
)
class TestImportXmlCommandCompletes(TestCase):

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def test_run_import_xml(self):
        fragment = """
        <item>
            <title>A title</title>
            <link>https://www.example.com/a-title</link>
            <description />
            <content:encoded />
            <excerpt:encoded />
            <wp:post_id>44221</wp:post_id>
            <wp:post_date>2015-05-21 15:00:31</wp:post_date>
            <wp:post_date_gmt>2015-05-21 19:00:31</wp:post_date_gmt>
            <wp:post_modified>2015-05-21 15:00:44</wp:post_modified>
            <wp:post_modified_gmt>2015-05-21 19:00:44</wp:post_modified_gmt>
            <wp:comment_status>open</wp:comment_status>
            <wp:ping_status>closed</wp:ping_status>
            <wp:post_name>a-title</wp:post_name>
            <wp:status>publish</wp:status>
            <wp:post_type>post</wp:post_type>
        </item>
        <item>
            <title>Not imported</title>
            <link>https://www.example.com/a-title</link>
            <description />
            <content:encoded />
            <excerpt:encoded />
            <wp:post_id>44221</wp:post_id>
            <wp:post_date>2015-05-21 15:00:31</wp:post_date>
            <wp:post_date_gmt>2015-05-21 19:00:31</wp:post_date_gmt>
            <wp:post_modified>2015-05-21 15:00:44</wp:post_modified>
            <wp:post_modified_gmt>2015-05-21 19:00:44</wp:post_modified_gmt>
            <wp:comment_status>open</wp:comment_status>
            <wp:ping_status>closed</wp:ping_status>
            <wp:post_name>a-title</wp:post_name>
            <wp:status>draft</wp:status>
            <wp:post_type>post</wp:post_type>
        </item>
        """
        built_file = generate_temporary_file(
            build_xml_stream(xml_items_fragment=fragment).read()
        )

        with self.assertRaises(FileNotFoundError):
            call_command(
                "import_xml",
                built_file,
                "2",
                "-a",
                "example",
                "-m",
                "TestPage",
                "-t",
                "post",
                "-s",
                "publish",
            )

        parent_page = Page.objects.get(id=2)
        imported_pages = parent_page.get_children().all()
        self.assertEqual(imported_pages.count(), 1)
        self.assertEqual(imported_pages[0].title, "A title")


class TestReduceCommand(TestCase):
    def test_without_arguments(self):
        with self.assertRaises(CommandError) as ctx:
            call_command("reduce_xml")
        self.assertEqual(
            str(ctx.exception),
            "Error: the following arguments are required: xml_file",
        )

    def test_xml_file_error(self):
        # The XML file does not exist
        with self.assertRaises(SystemExit):
            call_command("reduce_xml", "test.xml")

    def test_get_xml_file(self):
        cmd = ReduceCmd()
        with self.assertRaises(SystemExit):
            xml_file = cmd.get_xml_file("test.xml")
        with self.assertRaises(SystemExit):
            xml_file = cmd.get_xml_file("folder/test.xml")

        built_file = generate_temporary_file(
            build_xml_stream(xml_tags_fragment="").read()
        )
        self.assertEqual(built_file, cmd.get_xml_file(built_file))

    def test_xml_reduce_generate_stats(self):
        stats_filename = generate_stats_file("/folder/test.xml", {})
        self.assertEqual(stats_filename, "stats-test.xml.json")

        stats_filename = generate_stats_file("test.xml", {})
        self.assertEqual(stats_filename, "stats-test.xml.json")



from django.apps import apps
from django.core.management.base import BaseCommand
from wagtail.core.models import Page


class Command(BaseCommand):
    help = """Utilitly command to delete pages by type with optional limit to a parent page."""

    """
    When using sqlite3 the command line bulk delete won't work:
        on a site with a lot of pages so we'll fall back to looping

    ./manage.py delete_imported_xml app model [--parent_id]
    """

    def add_arguments(self, parser):
        parser.add_argument(
            "app",
            type=str,
            help="The app which contains your page models for the delete action",
            default="pages",
        )
        parser.add_argument(
            "model",
            type=str,
            help="A page model to use for the deleting pages",
            default="PostPage",
        )
        parser.add_argument(
            "-p",
            "--parent_id",
            type=int,
            help="The page ID of the parent page to use when deleting imported pages",
        )

    def handle(self, *args, **options):

        parent = None
        pages_to_delete = None

        try:
            page_model = apps.get_model(options["app"], options["model"])
        except LookupError:
            self.stdout.write(
                self.style.ERROR(
                    f"Cannot find the app and or model '{options['app']}.{options['model']}'"
                )
            )
            exit()

        if options["parent_id"]:
            # limit to delete pages of a page type to children of a parent page
            try:
                parent = Page.objects.get(id=options["parent_id"])
            except Page.DoesNotExist:
                self.stdout.write(
                    self.style.ERROR(
                        f"Cannot find the parent page with id = {options['parent_id']}"
                    )
                )
                exit()

        if not parent:
            pages_to_delete = page_model.objects.defer_streamfields().specific()
        else:
            self.stdout.write(
                self.style.WARNING(
                    f"Limiting delete action to children of page: [{parent.id}] '{parent.title}'"
                )
            )
            pages_to_delete = parent.get_children().exact_type(page_model)

        self.stdout.write("⏳ working ...")
        self.stdout.write(self.style.NOTICE(f"{len(pages_to_delete)} pages to delete"))

        for page in pages_to_delete:
            has_children = ""
            if page.get_children():
                has_children = " inc. child pages"
            self.stderr.write(
                self.style.SUCCESS(f"deleting [{page.id}] '{page.title}'{has_children}")
            )
            page.delete()



import json
import os
import xml.etree.ElementTree as ET

from django.core.management.base import BaseCommand


# register all namespaces to keep them when the modified xml is saved
def register_all_namespaces(filename):
    namespaces = dict([node for _, node in ET.iterparse(filename, events=["start-ns"])])
    for ns in namespaces:
        ET.register_namespace(ns, namespaces[ns])

def generate_stats_file(file_path, type_stats):
    stats_filename = f"stats-{file_path.split('/')[-1]}.json"
    f = open(stats_filename, "w")
    f.write(json.dumps(type_stats, indent=2))
    return stats_filename

class Command(BaseCommand):

    help = """Utils to reduce xml file size by removing unwanted tags."""

    """
    Tags to be removed item > wp:comment
    """

    def add_arguments(self, parser):
        parser.add_argument("xml_file", type=str, help="The full path to your xml file")

    def get_xml_file(self, xml_file):

        if os.path.exists(xml_file):
            return xml_file

        self.stdout.write(
            self.style.ERROR(f"The xml file `{xml_file}` cannot be found")
        )
        exit()

    def handle(self, *args, **options):
        file_path = self.get_xml_file(options["xml_file"])
        self.stdout.write(self.style.WARNING("Reducing ..."))
        self.stdout.write(
            self.style.NOTICE(
                "If this is a large file and has many depths it will take some time to complete"
            )
        )
        register_all_namespaces(file_path)

        num_lines_original = sum(1 for line in open(file_path))
        num_lines_original_formatted = "{:,}".format(num_lines_original)

        self.stdout.write(f"Original #lines {num_lines_original_formatted}")

        ofile = file_path.split(".")[0]
        output_file_name = f"{ofile}-reduced.xml"
        output_file_path = f"{output_file_name}"

        tree = ET.parse(file_path)
        wp = "{http://wordpress.org/export/1.2/}"
        items = tree.getroot()[0].findall("item")
        item_types = []
        item_statuses = []
        for item in items:
            comments = item.findall(f"{wp}comment")
            item_type = item.find(f"{wp}post_type").text
            item_status = item.find(f"{wp}status").text
            if item_type not in item_types:
                item_types.append(item_type)
            if item_status not in item_statuses:
                item_statuses.append(item_status)
            for comment in comments:
                item.remove(comment)

        tree.write(output_file_path)

        type_stats = [
            {
                type: len(
                    tree.getroot()[0].findall(f".//item/[{wp}post_type='{type}']")
                ),
                "status_counts": {
                    status: len(
                        tree.getroot()[0].findall(
                            f".//item/[{wp}post_type='{type}'][{wp}status='{status}']"
                        )
                    )
                    for status in item_statuses
                },
            }
            for type in item_types
        ]

        num_lines_out = sum(1 for line in open(output_file_path))
        num_lines_out_formatted = "{:,}".format(num_lines_out)

        num_lines_diff = num_lines_original - num_lines_out
        num_lines_diff_formatted = "{:,}".format(num_lines_diff)

        self.stdout.write(f"Output #lines {num_lines_out_formatted}")

        self.stdout.write(f"Saved  #lines {num_lines_diff_formatted}")

        self.stdout.write(self.style.WARNING(f"\nItem types of interest -------------"))
        type_list = ", ".join(item_types)
        self.stdout.write(f"\n[{type_list}]")

        self.stdout.write(self.style.WARNING(f"\nItem statuses -------------"))
        status_list = ", ".join(item_statuses)
        self.stdout.write(f"\n[{status_list}]")

        # stats to file
        stats_filename = generate_stats_file(file_path, type_stats)
        self.stdout.write(
            self.style.SUCCESS(f"\nStats file is here: {stats_filename}")
        )

        self.stdout.write(
            self.style.SUCCESS(f"\nFinished Your file is here: {output_file_name}")
        )
